function analyze_logs_plus_r6_SC(heights_filename,day_nums,tissue_nums, p_order, ...
    w_size, spike_thresh, peak_thresh, postrad, deflanalysis, asy, omitfr, inclfr, stime, etime, plot_raw,diastolicdist,save_ftfig,save_ftdata)
% Revised by Joshua Mayourian 4/14/20
% Revised by Serena Chang 5/28/21

% analyze_logs_plus then calculates force, stress and other parameters are calculated based on GUI inputs

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Defining analysis parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

post_radius = postrad; % uses post radius from GUI input
thresh = peak_thresh; % uses peak detection threshold from GUI input: increased threshold --> decreased peaks detected

% spike filter sensitivity from GUI: raise to decrease spike points discarded
if (spike_thresh == 0)
    sens = 10; % default value if box unchecked (uncommon to check)
else
    sens = 3 - spike_thresh*1.5;
end

capture_tol = 0.4; % tolerance in the FFT frequency that still allows a match for finding the max capture rate (in Hz); typically unchanged
num_tissues = 1; % analyze one tissue at a time (often at several pacing frequencies)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Defining plotting parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

figsize = [100, 100, 1200, 800]; % size of the plots that are drawn
plot_colors = {'b', 'c', 'k', 'g'}; % the progression of colors; if there are more plots than colors, then this will wrap around
y_margin = 0.025; % fraction of y-scale that is used as padding below and above the signals in the post deflection graphs
plot_ffts = false; % if set to true, FFTs will be plotted for each run
plot_peaks = false; % if set to true, all peaks will be plotted
plot_valleys = false; % if set to true, all valleys will be plotted

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Get all logs of interest for each tissue by
% looking up the available filenames
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

num_logs = [];
pacing_freqs = {};
highest_index = {}; % record the highest index found for each tissue number and pacing frequency (indexed by tissue number, then by pacing frequency)
highest_ind_pf = {}; % the pacing frequency for each index

for t=1:num_tissues
    highest_index{t} = [];
    highest_ind_pf{t} = [];
    % find all filenames in the format 't(tissue number) @ P hz XXX.dat'
    entries = struct2cell(dir());
    all_filenames = entries(1,:);
    % match strings with the defined filename format
    match_str = ['^T', num2str(tissue_nums(t)), ' @ [0-9]{1,3}(\.[0-9]{1,3}){0,1} hz [0-9]{3,3}\.dat$'];
    matched_filenames = regexp(all_filenames, match_str);
    matched_ind = find(cellfun('length', matched_filenames));
    pacing_freqs{t} = [];
    for ind = matched_ind
        % tokenize each filename string and get the third token - this is the pacing frequency
        tokens = strread(all_filenames{ind}, '%s');
        pfreq = sscanf(tokens{3}, '%f');
        % get the index and see if it is the highest one - if so, record it
        index = str2num(tokens{5}(1:3));
        pf_ind = find(highest_ind_pf{t} == pfreq);
        if (length(pf_ind) == 0);
            highest_index{t} = [highest_index{t}, index];
            highest_ind_pf{t} = [highest_ind_pf{t}, pfreq];
        elseif (highest_index{t}(pf_ind) < index);
            highest_index{t}(pf_ind) = index;
        end
    end
    % sort pacing frequencies
    omit_freqs = str2num(omitfr);
    pacing_freqs{t} = unique(highest_ind_pf{t});
    
    if isempty(inclfr)
        omitted_freqs = intersect(pacing_freqs{t}, omit_freqs);
        disp('Omitted Frequencies:  ');
        disp(omitted_freqs);
        pacing_freqs{t} = setdiff(pacing_freqs{t},omitted_freqs);
    else
        incl_freqs = str2num(inclfr);
        pacing_freqs{t} = intersect(pacing_freqs{t}, incl_freqs);
    end
    if (length(pacing_freqs{t}) == 0);
        error(['T', num2str(tissue_nums(t)), ': no logs found!']);
    end
    num_logs(t) = length(pacing_freqs{t});
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Get post and tissue heights from CSV file
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
heights_file = readtext([heights_filename,'.csv']);
% find the header row by finding the first row containing numerical data
header_row = -1;
for row=1:size(heights_file,1),
    if (ischar(heights_file{row,1}) == 0 && ...
            size(heights_file{row,1}, 1) ~= 0),
        header_row = row-1;
        break;
    end
end
if (header_row == -1),;
    error(['There is no numerical data in the ' ...
        ' first column of the heights table!']);
end
% strip header, convert to a matrix
heights_table = heights_file(header_row+1: size(heights_file,1),:);

if (size(heights_table, 2) ~= 8),;
    error(['Heights file needs to have eight columns']);
end
ht_tissue_nums = cell2mat(heights_table(:,1))';

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Parameters to find for each log in each tissue (indexed by tissue number)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

dev_forces = {}; % the maximum force exerted by the tissue (calculated using the maximum deflection)
cx_areas = {}; % cross-sectional area (lookup from metadata file)
exc_thresholds = {}; % excitation thresholds (V)
dev_stresses = {}; % maximum stress (max force  / cx area)
beating_freqs = {}; % beating frequencies
beating_rates = {};
beating_rates_std = {};
max_capture = []; % max capture rate for each tissue: the highest frequency at which the beating frequency is approximately equal to the pacing frequency
is_captured = {}; % true if a given run for a given tissue is captured, false if not used for normalizing axes for plotting
max_deflections = {}; % absolute max (no matter how the max deflections are calculated for finding the force)
max_times = {};

% Parameters to find for each tissue from the heights table (indexed by tissue number)
tissue_heights = [];
tissue_types = {};
type_nums = [];
t2post_data = {};

% Excitation thresholds from the heights table - use these if they don't exist in the log file itself
ht_exc_thresholds = [];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Here is where the analysis starts
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

for t=1:num_tissues
    
    %%%%%% Step 1: Find the tissue in the heights file %%%%%%
    tissue_entry = find(ht_tissue_nums == tissue_nums(t));
    if (length(tissue_entry) < 1)
        error(['Could not find tissue #' num2str(tissue_nums(t)) ...
            ' in heights table']);
    end
    if (length(tissue_entry) > 1)
        error(['More than one entries found for tissue #' ...
            num2str(tissue_nums(t)) ' in heights table']);
    end
    
    L_l = heights_table{tissue_entry, 2};
    a_l = heights_table{tissue_entry, 3};
    L_r = heights_table{tissue_entry, 4};
    a_r = heights_table{tissue_entry, 5};
    tissue_heights(t) = (a_l+a_r)/2;
    ht_exc_thresholds(t) = heights_table{tissue_entry, 6};
    type_nums(t) = heights_table{tissue_entry, 7};
    tissue_types{t} = heights_table{tissue_entry, 8};
    if (isstr(tissue_types{t}) == 0)
        tissue_types{t} = num2str(tissue_types{t});
    end
    
    
    %%%%%% Step 2: Display tissue properties from the heights file %%%%%%
    disp(['Tissue #' num2str(tissue_nums(t))]);
    disp(['Type: ', tissue_types{t}]);
    pfreq_str = [];
    for i=1:length(pacing_freqs{t})
        pfreq_str = [pfreq_str, num2str(pacing_freqs{t}(i))];
        if (i ~= length(pacing_freqs{t}));
            pfreq_str = [pfreq_str, ', '];
        end
    end
    disp(['Found pacing frequencies: ', pfreq_str]);
    disp(['Post height (left):    ' num2str(L_l)]);
    disp(['Tissue height (left):  ' num2str(a_l)]);
    disp(['Post height (right):   ' num2str(L_r)]);
    disp(['Tissue height (right): ' num2str(a_r)]);
    disp(['Excitation threshold (in heights table): ', ...
        num2str(ht_exc_thresholds(t))]);
    disp(sprintf('\n'));
    
    %%%%%% Step 3: Define variables of interest to obtain below %%%%%%
    dev_forces{t} = [];
    cx_areas{t} = [];
    exc_thresholds{t} = [];
    dev_stresses{t} = [];
    beating_freqs{t} = [];
    beating_rates{t} = [];
    beating_rates_std{t} = [];
    max_deflections{t} = [];
    max_times{t} = [];
    t2next = {};
    t2prev = {};
    defanvars{t} = [];
    measurement_times = {}; % the vector of time values for each log
    post_deflections = {};
    maxima = {};
    minima = {};
    max_t = {};
    min_t = {};
    raw_force = {};
    smooth_data = {};
    force = {};
    stress = {};
    
    %%%%%% Step 4: Analyze tissue logs %%%%%%
    for i=1:num_logs(t)
        
        %%%%%% Step 4.1: Select log to be analyzed %%%%%%
        pf_ind = find(highest_ind_pf{t} == pacing_freqs{t}(i));
        if (length(pf_ind) ~= 1)
            error('Something is very wrong!');
        end
        
        log_filename = ['T', num2str(tissue_nums(t)), ' @ ', ...
            num2str(pacing_freqs{t}(i)), ' hz ', ...
            sprintf('%03i', highest_index{t}(pf_ind))];
        log = dlmread([log_filename, '.dat']);
        
        %%%%%% Step 4.2: Clean up data (measurement times and post distances) %%%%%%
        measurement_times{i} = log(:,1) - min(log(:,1));
        
        indices = [];
        if (etime > -1)
            indices = find(measurement_times{i} > stime & measurement_times{i} < etime);
        elseif (stime>0)
            indices = find(measurement_times{i} > stime);
        end
        
        %post_distances = log(:,4)-log(:,2);
        post_distances = sqrt(((log(:,4)-log(:,2)).^2 + ((log(:,5)-log(:,3)).^2))); % define post distance as difference between L and R posts, accounting for both X and Y
        
        if ~isempty(indices)
            measurement_times{i} = measurement_times{i}(indices);
            post_distances = post_distances(indices);
        end
        
        durtime{i} = max(measurement_times{i})-min(measurement_times{i}); %duration of recording in analysis
        
        max_times{t}(i) = max(measurement_times{i});
        
        %%%%%% Step 4.3: Get data from metafile %%%%%%
        tokens = textread([log_filename '.txt'], '%s');
        cx_token1 = 0;
        et_token = 0;
        e_spacing_token = 0;
        for j=1:length(tokens)
            % look for cross-sectional areas
            if strcmp(tokens{j}, '(mm^2):')  == 1
                % three found
                if strcmp(tokens{j+2}, 'Pacing') ~= 1
                    cx_token1 = j+1;
                    cx_token2 = j+2;
                    cx_token3 = j+3;
                    % for the first two area tokens, we need to strip the trailing comma
                    cx_area1 = str2num(tokens{cx_token1}(1:length(tokens{cx_token1})-1));
                    cx_area2 = str2num(tokens{cx_token2}(1:length(tokens{cx_token2})-1));
                    cx_area3 = str2num(tokens{cx_token3});
                    % average the cross-sectional areas together
                    cx_areas{t}(i) = mean([cx_area1 cx_area2 cx_area3]);
                    % one found
                else
                    cx_token1 = j+1;
                    cx_area1 = str2num(tokens{cx_token1});
                    cx_areas{t}(i) = cx_area1;
                end
            end
            % look for excitation threshold and electrode spacing
            if (strcmp(tokens{j}, '(V):') && ...;
                    strcmp(tokens{j-2}, 'Excitation') && ...;
                    strcmp(tokens{j-1}, 'threshold'));
                et_token = j+1;
            end
            if (strcmp(tokens{j}, '(mm):'));
                e_spacing_token = j+1;
            end
        end
        % no cx area found or is zero? Ask for it
        tissue_name = ['T', num2str(tissue_nums(t)), ' (', ...
            num2str(tissue_types{t}), ') @ ', ...
            num2str(pacing_freqs{t}(i)), ' Hz'];
        if (cx_token1 == 0 || (cx_token1 ~= 0 && cx_area1 == 0)),
            cx_area1 = input([tissue_name, ' cx area 1 (mm^2)? ']);
            cx_area2 = input([tissue_name, ' cx area 2 (mm^2)? ']);
            cx_area3 = input([tissue_name, ' cx area 3 (mm^2)? ']);
            cx_areas{t}(i) = mean([cx_area1 cx_area2 cx_area3]);
        end
        % If we have ET and electrode spacing, save that; otherwise,
        % use the value that was taken from the table
        if (et_token ~= 0 && e_spacing_token ~= 0 && ...
                ~strcmp(tokens{et_token}, 'ND') && ...
                str2num(tokens{e_spacing_token}) ~= 0);
            et_voltage = str2num(tokens{et_token});
            e_spacing = str2num(tokens{e_spacing_token});
            exc_thresholds{t}(i) = et_voltage / e_spacing;
        else
            exc_thresholds{t}(i) = ht_exc_thresholds(t);
        end
        
        %%%%%% Step 4.4: Start calculations %%%%%%
        
        %%% Step 4.4.1: Raw force calcs %%%
        assignin('base','post_distances', post_distances)
        total_deflections = diastolicdist - post_distances;
        f_k = a_r^2/a_l^2 * (3*L_r - a_r)/(3*L_l - a_l); % accounts for L vs R height differences
        post_deflections{i} = total_deflections / (1 + f_k);
        
        % Find the beating frequency by doing an FFT of the post deflection
        raw_force{i} = post_force2(post_deflections{i}, L_l, a_l, post_radius); % calculate force
        smooth_data{i} = sgolayfilt(raw_force{i},p_order,w_size); % smooths force data
        
        [smooth_data{i} measurement_times{i} indout] = removeoutliers2(smooth_data{i},measurement_times{i},sens);
        raw_force{i} = raw_force{i}(indout); % Raw force after outliers
        post1def = log(indout,4);
        post2def = log(indout,2);
        assignin('base','filt_force',smooth_data{i});
        assignin('base','time',measurement_times{i});
        
        %%% Step 4.4.2: Beating freq calcs %%%
        sampling_freq = 1 / mean(diff(measurement_times{i}));
        % make deflections zero-mean to remove bias
        zm_force = smooth_data{i} - mean(smooth_data{i});
        % the rest: shamelessly ripped from
        % http://www.mathworks.com/access/helpdesk/help/techdoc/ref/fft.html
        L = length(measurement_times{i});
        NFFT = 2^nextpow2(L);
        fc = fft(zm_force, NFFT)/L; % frequency components
        fc = 2*abs(fc(1:NFFT/2+1)); % make components absolute, only use absolute
        f = sampling_freq/2 * linspace(0, 1, NFFT/2+1);
        % find max frequency
        [max_fc max_i] = max(fc);
        beating_freqs{t}(i) = f(max_i);
        
        %%% Step 4.4.3: Find mins/maxs to calc devF %%%
        [max_t{i}, maxima{i}, min_t{i}, minima{i}] = find_pv7(measurement_times{i}, smooth_data{i},thresh);
        %???????????????????????VERIFY WHAT THIS LINE IS FOR ????????????????????
        [max_ind{i}, maxjunk, min_ind{i}, minjunk] = find_pv7(1:length(smooth_data{i}-1), smooth_data{i},thresh);
        clearvars maxjunk minjunk
        
        assignin('base','max_t',max_t);
        assignin('base','maxima',maxima);
        assignin('base','min_t',min_t);
        assignin('base','minima',minima);
        
        peak_v{i} = zeros(size(smooth_data{i}));
        val_v{i} = zeros(size(smooth_data{i}));
        cur_pk = 1;
        cur_vl = 1;
        
        for n=1:length(smooth_data{i})
            
            if ~isempty(maxima{i})
                peak_v{i}(n) = maxima{i}(cur_pk);
                val_v{i}(n) = minima{i}(cur_vl);
                
                if (cur_pk < length(maxima{i}) && measurement_times{i}(n) >= max_t{i}(cur_pk+1));
                    cur_pk = cur_pk + 1;
                end
                if (cur_vl < length(minima{i}) && measurement_times{i}(n) >= min_t{i}(cur_vl+1));
                    cur_vl = cur_vl + 1;
                end
                
            end
        end
        
        if (~isempty(maxima{i}))
            %%% Step 4.4.4: Calc devF, sysF, stresses, beat rate  %%%
            
            max_deflections{t}(i) = max(maxima{i});
            
            dev_forces{t}(i) = mean(peak_v{i}-val_v{i});
            dev_forc_std{t}(i) = std(peak_v{i}-val_v{i});
            % max stress = max force / area (units: mN / mm^2 = kPa)
            dev_stresses{t}(i) = dev_forces{t}(i) / cx_areas{t}(i);
            dev_stress_std{t}(i) = dev_forc_std{t}(i)/cx_areas{t}(i);
            %%%%%%%%%%%%%% end additional processing
            dias_forces{t}(i) = mean(val_v{i});
            dias_forc_st{t}(i) = std(val_v{i});
            syst_forces{t}(i) = mean(peak_v{i});
            syst_forces_st{t}(i) = std(peak_v{i});
            
            force{t}{i}(:,1) = measurement_times{i};
            force{t}{i}(:,2) = smooth_data{i};
            stress{t}{i}(:,1) = measurement_times{i};
            stress{t}{i}(:,2) = smooth_data{i} / cx_areas{t}(i);
            
            
            % calculate beating rates (event based contraction rate).
            periods = diff(max_t{i});
            beatrate = 1./periods;
            
            assignin('base','peaktimes',max_t{i});
            assignin('base','periods',periods);
            assignin('base','beatrates',beatrate);
            beating_rates{t}(i) = mean(beatrate);
            beating_rates_std{t}(i) = std(beatrate);
            
            t2prev{i} = periods(1:end-1);
            t2next{i} = periods(2:end);
            
        end
        
        %Save force data
        if save_ftdata
            temptable = force{t}{i};
            tablename = [heights_filename '_T' num2str(tissue_nums(t)) '_' num2str(i) '_forcetimedata.txt'];
            tableheader= {'Time' 'Data'};
            tblhdr = sprintf('%s\t',tableheader{:});
            tblhdr(end) = '';
            dlmwrite(tablename,tblhdr,'');
            dlmwrite(tablename,temptable,'-append','delimiter','\t');
        end
        
        
        %%% Step 4.4.5: Calc slopes  %%%
        %find the average slopes of contraction and relaxation
        contractpoints{i} = {};
        relaxpoints{i} = {};
        %contraction_times = zeros(length(maxima),3);
        contraction_times{i} = [];
        h = waitbar(0, 'Finding slope points');
        lastl = [];
        lastr = [];
        r = 1;
        relaxedpoints = [];
        
        T50vals{i} = [];
        time = measurement_times{i};
        
        for z = 1:length(maxima{i})-1;
            Tl = [];
            Tr = [];
            waitbar(z/length(maxima{i}));
            
            %floor of peak is average of min on either side of the peak
            minimum = (val_v{i}(max_ind{i}(z))+val_v{i}(max_ind{i}(z+1)))/2;
            
            %get bounds for 10 to 90 percent developed force
            F90 = (maxima{i}(z) - minimum)*.90 + minimum;
            F10 = (maxima{i}(z) - minimum)*.10 + minimum;
            F50 = (maxima{i}(z) + minimum)/2;
            
            %set pointers to go down contraction and relaxation slopes
            llimit = r;
            rlimit = max_ind{i}(z+1);
            l = max_ind{i}(z)-1;
            r = max_ind{i}(z)+1;
            
            %empty arrays for storing the slope points between 10 and 90 percent
            relaxpoints{i}{z,1} = max_ind{i}(z);
            contractpoints{i}{z,1} = max_ind{i}(z);
            relaxpoints{i}{z,2} = [];
            contractpoints{i}{z,2} = [];
            tempr = [];
            templ = [];
            
            %find and store points for relax slope fitting
            while smooth_data{i}(r) > F10 && r<rlimit;
                if (smooth_data{i}(r)<F90);
                    relaxpoints{i}{z,2} = [relaxpoints{i}{z,2} r];
                    tempr = [tempr r];
                    %find T50 right bound
                    if (smooth_data{i}(r)<F50 && isempty(Tr));
                        Tr = r;
                    end
                    r = r+1;
                else
                    r = r+1;
                end
            end
            
            %find and store points for contraction slope fitting
            while  smooth_data{i}(l) > F10 && l>llimit;
                if (smooth_data{i}(l)<F90);
                    contractpoints{i}{z,2} = [l contractpoints{i}{z,2}];
                    templ = [l templ];
                    %find T50 left bound
                    if (smooth_data{i}(l)<F50 && isempty(Tl));
                        Tl = l;
                    end
                    l = l-1;
                else
                    l = l-1;
                end
            end
            
            %%% Step 4.4.6: Calc T50, time to contraction, time to relaxation  %%%
            if ~isempty(Tl) && ~isempty(Tr);
                XL = smooth_data{i}(Tl:Tl+1);
                XR = smooth_data{i}(Tr-1:Tr);
                Tlx = time(Tl:Tl+1);
                Trx = time(Tr-1:Tr);
                a=1;
                b=2;
                while (XR(1)==XR(2));
                    XL = [smooth_data{i}(Tl-a),smooth_data{i}(Tl+b)];
                    Tlx = [time(Tl-a),time(Tl+b)];
                    a=a+1;
                    b=b+1;
                end
                a=1;
                b=2;
                while (XL(1) == XL(2));
                    XR = [smooth_data{i}(Tr-b),smooth_data{i}(Tr+a)];
                    Trx = [time(Tr-b),time(Tr+a)];
                    a=a+1;
                    b=b+1;
                end
                T50l = interp1(XL,Tlx,F50);
                T50r = interp1(XR,Trx,F50);
                T50vals{i}(end+1,:) = [time(max_ind{i}(z)),T50r-T50l,F50,time(max_ind{i}(z))-T50l,T50r - time(max_ind{i}(z))];
                
                %Find time to peak and time to relaxation
                xl = smooth_data{i}(l:l+1);
                xr = smooth_data{i}(r-1:r);
                tl = time(l:l+1);
                tr = time(r-1:r);
                
                relaxedpoints = [xl xr];
                
                if length(unique(xl))>1;
                    t10_start = interp1(xl,tl,F10);
                else
                    t10_start = mean(tl);
                end
                if length(unique(xr))>1;
                    t10_end = interp1(xr,tr,F10);
                else
                    t10_end = mean(tr);
                end
                
                t2peak =  time(max_ind{i}(z))-t10_start;
                t2relax90 = t10_end - time(max_ind{i}(z));
                t2relax50 = T50r - time(max_ind{i}(z));
                
                if t2peak < 6 && t2relax90 < 6;
                    contraction_times{i}(end+1,:) = [time(max_ind{i}(z)) t2peak t2relax50 t2relax90];
                    
                else
                    fprintf(['Outlier found, T2Pk: %.1f T2Rx:  %.1f, Time:  %.1f\n'],...
                        t10_start, t10_end, time(max_ind{i}(z)));
                end
            end
            
        end
        close(h)
        assignin('base', 'contraction_times',contraction_times);
        assignin('base', 'T50vals', T50vals);
        assignin('base', 'relaxedpoints', relaxedpoints);
        assignin('base','contractpoints',contractpoints);
        assignin('base','relaxpoints',relaxpoints);
        assignin('base','times',measurement_times);
        assignin('base','raw_force',raw_force);
        
        if deflanalysis
            tname = ['T', num2str(tissue_nums(t)), ' (', ...
                num2str(tissue_types{t}), ') @ ', ...
                num2str(pacing_freqs{t}(i)), ' Hz'];
            [davars t2post_temp] = analyzePostDefl(post1def,post2def,measurement_times{i},thresh,p_order,w_size,tname);
            defanvars{t}(i,:) = davars;
            t2post_data{t}(i,:) = t2post_temp;
            assignin('base','postdefvars', defanvars);
            assignin('base','t2postdata', t2post_data);
        end
        
        
    end
    
    assignin('base','Force',force);
    assignin('base','peaks',peak_v);
    assignin('base','trough',val_v);
    assignin('base','Stress',stress);
    assignin('base','t2prev', t2prev);
    assignin('base','t2next', t2next);
    
    
    is_captured{t} = (pacing_freqs{t} >= ...;
        (beating_freqs{t} - capture_tol)) & ...;
        (pacing_freqs{t} <= ...;
        (beating_freqs{t} + capture_tol));
    captured_inds = find(is_captured{t});
    if (length(captured_inds) ~= 0);
        max_capture(t) = max(beating_freqs{t}(captured_inds));
    else
        disp(['Tissue #',num2str(tissue_nums(t)), ...
            ': no captured frequencies found! Setting '...
            'max capture to 0 Hz']);
        max_capture(t) = 0;
    end
    
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Plotting
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    % increase figure number because we previously plotted stuff
    fign = 1;
    if (deflanalysis==1)
        fign = i+1;
    end
    if (plot_peaks == true || plot_valleys == true);
        fign = fign + 1;
    end
    
    % figure out the layout of the plot from the the number of logs
    if (num_logs(t) == 1),;
        rows = 1;
        cols = 1;
    elseif (num_logs(t) == 2),;
        rows = 1;
        cols = 2;
    elseif (num_logs(t) == 3),;
        rows = 1;
        cols = 3;
    elseif (num_logs(t) == 4),;
        rows = 2;
        cols = 2;
    elseif (num_logs(t) == 5),;
        rows = 3;
        cols = 2;
    elseif (num_logs(t) == 6),;
        rows = 3;
        cols = 2;
    elseif (num_logs(t) == 7),;
        rows = 4;
        cols = 2;
    else
        rows = 4;
        cols = 2;
    end
    
    pnmax = rows*cols;
    % figure out the scale
    x_max = max(max_times{t});
    y_max = max(max_deflections{t});
    
    pn = 1; % subplot number
    testq = 0; % cue second figure being saved if value = 1
    for i=1:num_logs(t),;
        figure(fign);
        set(gcf,'Position',figsize);
        subplot(rows, cols, pn);
        pn = pn + 1;
        if (pn > pnmax);
            pn = 1;
            fign = fign + 1;
            testq = 1; %number to cue saving of the second figure if needed
        end
        
        if plot_raw
            plot(measurement_times{i}, raw_force{i},'b');
        end
        
        hold on
        plot(measurement_times{i}, smooth_data{i}, ...
            plot_colors{mod(i-1, length(plot_colors)) + 1}, ...
            max_t{i}, maxima{i}, 'm.-', ...
            min_t{i}, minima{i}, 'c.-');
        
        plot(measurement_times{i},smooth_data{i},'r')
        xlabel('Time (sec)');
        ylabel('Force (mN)');
        tissue_name = ['T', num2str(tissue_nums(t)), ' (', ...
            num2str(tissue_types{t}), ') @ ', ...
            num2str(pacing_freqs{t}(i)), ' Hz'];
        title(tissue_name);
        figname = [heights_filename '_T' num2str(tissue_nums(t))];
        figname2 = [heights_filename '_T' num2str(tissue_nums(t)) '_1'];
        %axis([0 x_max -y_margin*y_max (1+y_margin)*y_max]);
        
        if save_ftfig
            if testq == 0
            saveas(gcf,figname,'fig');
            else
                saveas(gcf,figname2,'fig');
            end
        end  
    end
    

    fign = fign + 1;
    
    
    % plot a zoomed-in picture of the deflections
    % find the highest pacing period (inverse of lowest non-zero frequency)
    highest_period = 1 / min(pacing_freqs{t}(find(pacing_freqs{t})));
    hands = [];
    pn = 1;
    for i=1:num_logs(t),
        figure(fign);
        set(gcf,'Position',figsize);
        hands(i)=subplot(rows, cols, pn);
        pn = pn + 1;
        if (pn > pnmax)
            pn = 1;
            fign = fign + 1;
        end
        plot(measurement_times{i}, smooth_data{i}, ...
            plot_colors{mod(i-1, length(plot_colors)) + 1});
        
        xlabel('Time (sec)');
        ylabel('Force (mN)');
        tissue_name = ['T', num2str(tissue_nums(t)), ' (', ...
            num2str(tissue_types{t}), ') @ ', ...
            num2str(pacing_freqs{t}(i)), ' Hz'];
        title(tissue_name);
        if (asy == 1)
            axis([0 x_max ...
                -inf inf]);
            % else
            %axis([0 x_max ...
            %   -y_margin*y_max (1+y_margin)*y_max]);
        end
        % plot the FFTs if requested
        if (plot_ffts == true)
            figure(fign);
            set(gcf,'Position',figsize);
            subplot(rows, cols, pn);
            pn = pn + 1;
            if (pn > pnmax)
                pn = 1;
                fign = fign + 1;
            end
            
            % the actual FFT code is a copy of what is seen earlier in
            % the script
            sampling_freq = 1 / mean(diff(measurement_times{i}));
            % make deflections zero-mean to remove bias
            zm_deflections = smooth_data{i} - mean(smooth_data{i});
            % the rest: shamelessly ripped from
            % http://www.mathworks.com/access/helpdesk/help/techdoc/ref/fft.html
            L = length(measurement_times{i});
            NFFT = 2^nextpow2(L);
            fc = fft(zm_deflections, NFFT)/L; % frequency components
            fc = 2*abs(fc(1:NFFT/2+1)); % make components absolute, only use absolute
            f = sampling_freq/2 * linspace(0, 1, NFFT/2+1);
            plot(f,fc);
            xlim([0, 20]);
            tissue_name = ['T', num2str(tissue_nums(t)), ' (', ...
                num2str(tissue_types{t}), ') @ ', ...
                num2str(pacing_freqs{t}(i)), ' Hz'];
            title(['FFT: ', tissue_name]);
            xlabel('Frequency (Hz)');
            ylabel('Intensity (arb)');
            
        else
            figure(fign);
            set(gcf,'Position',figsize);
            subplot(rows, cols, pn);
            pn = pn + 1;
            if (pn > pnmax)
                pn = 1;
                fign = fign + 1;
            end
            hold on;
            m=1.05*max([t2prev{i};t2next{i}]);
            plot(0:.1:m,0:.1:m,'k')
            plot(t2prev{i},t2next{i},'b*')
            period = 1/pacing_freqs{t}(i);
            plot(period,period, 'go')
            title (['Time to next peak vs time to previous peak ' tissue_name] )
            xlabel('Time to previous peak (s)')
            ylabel('Time to next peak (s)')
        end
    end
    
    fign = fign + 1;
    
    % plot the dev force as a function of the pacing frequency
    figure(fign);
    fign = fign + 1;
    set(gcf,'Position',figsize);
    subplot(3,1,1);
    plot(pacing_freqs{t}, dev_forces{t}, 'bo-');
    xlabel('Pacing frequency (Hz)')
    ylabel('Developed force (mN)')
    tissue_name = ['T', num2str(tissue_nums(t)), ' (', ...
        num2str(tissue_types{t}), ')'];
    title([tissue_name, ': Dev. force vs. pacing frequency']);
    % plot the max force as a function of the beating frequency
    subplot(3,1,2);
    % get the captured and uncaptured points and plot them separately
    captured_inds = find(is_captured{t});
    uncaptured_inds = find(~is_captured{t});
    hold on
    plot(beating_freqs{t}(captured_inds), ...
        dev_forces{t}(captured_inds), 'bo');
    plot(beating_freqs{t}(uncaptured_inds), ...
        dev_forces{t}(uncaptured_inds), 'ro');
    hold off
    xlabel('Detected beating frequency (Hz)');
    ylabel('Developed force (mN)')
    title([tissue_name, ': max force vs. beating (FFT) frequency']);
    % plot the beating frequency as a function of the pacing frequency
    subplot(3,1,3);
    plot(pacing_freqs{t}, beating_freqs{t}, 'bo-');
    xlabel('Pacing frequency (Hz)')
    ylabel('Detected beating frequency (Hz)');
    title([tissue_name, ': beating frequency vs. pacing frequency']);
end

slopes = analyze_slopes(measurement_times,smooth_data,contractpoints,relaxpoints,1,hands);
assignin('base','slopes',slopes);

%datatable = zeros(num_logs(t),41);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Output to data table
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

for i = 1:num_logs(t)
    
    datatable{i,1} = day_nums(t);
    datatable{i,2} = type_nums(t);
    datatable{i,3} = string(tissue_types(t));
    datatable{i,4} = tissue_nums(t);
    datatable{i,5} = tissue_heights(t);
    datatable{i,6} = cx_areas{t}(i);
    datatable{i,7} = exc_thresholds{t}(i);
    datatable{i,8} = max_capture(t);
    
    datatable{i,9} = pacing_freqs{t}(i);
    datatable{i,10} = beating_freqs{t}(i);
    datatable{i,11} = beating_rates{t}(i);
    datatable{i,12} = beating_rates_std{t}(i);
    datatable{i,13} = beating_rates_std{t}(i)/beating_rates{t}(i);
    
    datatable{i,14} = dias_forces{t}(i);
    datatable{i,15} = dias_forc_st{t}(i);
    datatable{i,16} = syst_forces{t}(i);
    datatable{i,17} = syst_forces_st{t}(i);
    datatable{i,18} = dev_forces{t}(i);
    datatable{i,19} = dev_forc_std{t}(i);
    datatable{i,20} = dev_stresses{t}(i);
    datatable{i,21} = dev_stress_std{t}(i);
    
    if ~isempty(T50vals{i})
        
        datatable{i,22} = mean(nonzeros(T50vals{i}(:,2)));
        datatable{i,23} = std(nonzeros(T50vals{i}(:,2)));
        datatable{i,24} = mean(nonzeros(T50vals{i}(:,4)));
        datatable{i,25} =  std(nonzeros(T50vals{i}(:,4)));
        datatable{i,26} =  mean(nonzeros(T50vals{i}(:,5)));
        datatable{i,27} =  std(nonzeros(T50vals{i}(:,5)));
        % datatable{i,28} = datatable(i,19)*beating_freqs{1}(i);
        datatable{i,28} = dev_forc_std{t}(i)*beating_freqs{1}(i);
        
    end
    if (~isempty(contraction_times{i}));
        
        datatable{i,29} = mean(nonzeros(contraction_times{i}(:,2)));
        datatable{i,30} = std(nonzeros(contraction_times{i}(:,2)));
        datatable{i,31} = mean(nonzeros(contraction_times{i}(:,3)));
        datatable{i,32} = std(nonzeros(contraction_times{i}(:,3)));
        datatable{i,33} = mean(nonzeros(contraction_times{i}(:,4)));
        datatable{i,34} = std(nonzeros(contraction_times{i}(:,4)));
    else
        
        datatable{i,29:34} = NaN;
    end
    
    if (~isempty(slopes{i}))
        riseslopes = slopes{i}(:,1);
        fallslopes = slopes{i}(:,3);
        datatable{i,35} = mean(riseslopes(isfinite(riseslopes)));
        datatable{i,36} = std(riseslopes(isfinite(riseslopes)));
        datatable{i,37} = mean(fallslopes(isfinite(fallslopes)));
        datatable{i,38}= std(fallslopes(isfinite(fallslopes)));
    end
    
    datatable{i,39} = cell2mat(durtime(i));
    datatable{i,40} = min(measurement_times{i});
    datatable{i,41} = max(measurement_times{i});
    
    datatable{i,42} = diastolicdist; 
datatable{i,43} = postrad;
datatable{i,44} = thresh;
datatable{i,45} = p_order;
datatable{i,46} = w_size;
datatable{i,47} = spike_thresh;
    
end


header = {'Day' 'Type' 'TissueType' 'Tissue' 'Height' 'CxArea' 'ExcThresholds' 'MaxCapture'...
    'PacingFreq' 'BeatingFreq' 'BeatRate' 'BeatRateSTD' 'BeatRateCOV' 'DiasForce_mean' 'DiasForce_std' 'SystForce_mean' 'SystForce_std' 'DevForce_mean' 'DevForce_std' 'DevStress_mean'...
    'DevStress_std' 'T50_mean' 'T50_std' 'C50_mean' 'C50_std' 'R50_mean' 'R50_std' 'P50' 'T2Pk_mean' 'T2Pk_std' 'T2Relx50_mean' 'T2Relx50_std' 'T2Rlx90_mean'...
    'T2Relx90_std' 'posdFdt_mean' 'posdFdt_std' 'negdFdt_mean' 'negdFdt_std' 'Duration' 'Start_Time' 'End_Time' 'DiasDist' 'PostRad' 'PeakDet' 'SGPoly' 'SGFrameSize' 'SpikeRemoval'};

fname = [heights_filename '_T' num2str(tissue_nums(t)) '_datatable.txt'];
% hdr = sprintf('%s\t',header{:});
% hdr(end) = '';

outputtab = cell2table(datatable, 'VariableNames', header);

if isfile(fname) == 1
    prompt = {'File exists. Enter number for new filename subscript: '};
    answer = inputdlg(prompt, 'New Filename');
    fname = [heights_filename '_T' num2str(tissue_nums(t)) '_datatable_' char(answer) '.txt'];
end

writetable(outputtab,fname,'delimiter','\t');
% dlmwrite(fname,hdr,'');
% dlmwrite(fname,datatable,'-append','delimiter','\t');
type(fname)

if (deflanalysis == 1)
    f2name = [heights_filename '_T' num2str(tissue_nums(t))  '_2datatable.txt'];
    header2= {'Qt' 'coher_idx', 'qt', 'symm_idx', 'MSD_L', 'MSD_R', 'SDSD_L', 'SDSD_R', 'LRMPD','LRSPD', 'MNN_L', 'MNN_R', 'SDNN_L', 'SDNN_R'};
    hdr2 = sprintf('%s\t',header2{:});
    hdr2(end) = '';
    if isfile(f2name) == 1
        prompt = {'File exists. Enter number for new filename subscript: '};
        answer = inputdlg(prompt, 'New Filename');
        f2name = [heights_filename '_T' num2str(tissue_nums(t)) '_2datatable_' char(answer) '.txt'];
    end
    dlmwrite(f2name,hdr2,'');
    dlmwrite(f2name,defanvars{1},'-append','delimiter','\t');
    type(f2name)
end

end
