function [peaks_x, peaks_y, valleys_x, valleys_y] = ...
    find_pv7(x, y,thresh)
% find_pv7: finds peaks and valleys in a signal and returns their x
% and y values. 

% Algorithm: The peaks and valleys are initially found using MATLAB's
% findpeaks() function, which simply finds local maxima, with a
% threshold setting that defines the minimum distance between peaks. A
% polynomial fit is then performed on each peak and valley if the
% poly_fit parameter is set to true. To perform the fit, the halfway
% line is found between the mean of the peaks and mean of the
% valleys. For each peak all points above the halfway line around the
% peak (likewise, for valleys, all points below the halfway line
% around the valley) are used in the polynomial fit. If there are not
% enough points to define a curve (polynomial order + 1) then the fit
% is not performed. The max/min of that polynomial then becomes the
% new peak/valley.
%
% Usage: [peaks_x, peaks_y, valleys_x, valleys_y] = 
%            find_pv(x, y, peak_distance, show_progress)
% where x and y are the x- and y-values of the data and peak_distance
% is the minimum distance allowed between successive peaks/valleys.
% If show_progress is set to true, a progress bar will be shown for
% the fitting of the peaks and valleys.

%determine threshold

ystd = [thresh*std(y) 1];
ystd(end+1,2) = length(y);

delta = ystd;

% initial finding of peaks and valleys - these will later be
% refined through polynomial fitting

[maxes mins] = peakdet2(y, delta);
max_ind = maxes(:,1);
max_vals = maxes(:,2);
min_ind = mins(:,1);
min_vals = mins(:,2);

max_corrected = max_vals;
min_corrected = min_vals;
max_i = find(ismember(max_vals,max_corrected));
min_i = find(ismember(min_vals,min_corrected));

max_vals = max_vals(max_i);
min_vals = min_vals(min_i);
max_ind = max_ind(max_i);
min_ind = min_ind(min_i);

% find halfway points between the peaks. Algorithm: make vectors
% containing the same number of points as the measurements and fill
% them out with the nearest peak/valley value. That way, the
% halfway point can be figured out at any given point by averaging
% the peak/valley vectors.

% cut off first and last peak/valley because these can be incorrect
% due to truncation of the signal

if (min_ind(1) < max_ind(1))
  min_ind = min_ind(2:end);
  min_vals = min_vals(2:end);
else
  max_ind = max_ind(2:end);
  max_vals = max_vals(2:end);
end
if (min_ind(end) > max_ind(end))
  min_ind = min_ind(1:end-1);
  min_vals = min_vals(1:end-1);
else
  max_ind = max_ind(1:end-1);
  max_vals = max_vals(1:end-1);
end

  peaks_x = x(max_ind);
  peaks_y = max_vals;
  valleys_x = x(min_ind);
  valleys_y = min_vals;

end
